"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Notification_1 = require("rxjs/Notification");
var Observable_1 = require("rxjs/Observable");
var VirtualTimeScheduler_1 = require("rxjs/scheduler/VirtualTimeScheduler");
var VirtualTimeScheduler_2 = require("rxjs/scheduler/VirtualTimeScheduler");
var ColdObservable_1 = require("rxjs/testing/ColdObservable");
var HotObservable_1 = require("rxjs/testing/HotObservable");
var parseObservableMarble_1 = require("../marbles/parseObservableMarble");
var SubscriptionMarbleToken_1 = require("../marbles/SubscriptionMarbleToken");
var TestMessage_1 = require("../message/TestMessage");
var calculateSubscriptionFrame_1 = require("./calculateSubscriptionFrame");
/**
 * @internal
 */
var TestScheduler = (function (_super) {
    tslib_1.__extends(TestScheduler, _super);
    function TestScheduler(autoFlush, frameTimeFactor) {
        if (autoFlush === void 0) { autoFlush = false; }
        if (frameTimeFactor === void 0) { frameTimeFactor = 1; }
        var _this = _super.call(this, VirtualTimeScheduler_2.VirtualAction, Number.POSITIVE_INFINITY) || this;
        _this.autoFlush = autoFlush;
        _this.frameTimeFactor = frameTimeFactor;
        _this.coldObservables = [];
        _this.hotObservables = [];
        _this.flushed = false;
        return _this;
    }
    TestScheduler.prototype.flush = function () {
        var hotObservables = this.hotObservables;
        while (hotObservables.length > 0) {
            hotObservables.shift().setup();
        }
        _super.prototype.flush.call(this);
        this.flushed = true;
    };
    TestScheduler.prototype.getMessages = function (observable, unsubscriptionMarbles) {
        var _this = this;
        if (unsubscriptionMarbles === void 0) { unsubscriptionMarbles = null; }
        var _a = calculateSubscriptionFrame_1.calculateSubscriptionFrame(observable, unsubscriptionMarbles, this.frameTimeFactor), subscribedFrame = _a.subscribedFrame, unsubscribedFrame = _a.unsubscribedFrame;
        var observableMetadata = [];
        var pushMetadata = function (notification) {
            return observableMetadata.push(new TestMessage_1.TestMessageValue(_this.frame, notification));
        };
        var subscription = null;
        this.schedule(function () {
            subscription = observable.subscribe(function (value) {
                return pushMetadata(Notification_1.Notification.createNext(value instanceof Observable_1.Observable ? _this.materializeInnerObservable(value, _this.frame) : value));
            }, function (err) { return pushMetadata(Notification_1.Notification.createError(err)); }, function () { return pushMetadata(Notification_1.Notification.createComplete()); });
        }, subscribedFrame);
        if (unsubscribedFrame !== Number.POSITIVE_INFINITY) {
            this.schedule(function () { return subscription.unsubscribe(); }, unsubscribedFrame);
        }
        if (this.autoFlush) {
            if (this.flushed) {
                throw new Error("Cannot schedule to get marbles, scheduler's already flushed");
            }
            this.flush();
        }
        return observableMetadata;
    };
    TestScheduler.prototype.createColdObservable = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var marbleValue = args[0], value = args[1], error = args[2];
        if (typeof marbleValue === 'string' && marbleValue.indexOf(SubscriptionMarbleToken_1.SubscriptionMarbleToken.SUBSCRIBE) !== -1) {
            throw new Error("Cold observable cannot have subscription offset " + SubscriptionMarbleToken_1.SubscriptionMarbleToken.SUBSCRIBE);
        }
        var messages = Array.isArray(marbleValue)
            ? marbleValue
            : parseObservableMarble_1.parseObservableMarble(marbleValue, value, error, false, this.frameTimeFactor);
        var observable = new ColdObservable_1.ColdObservable(messages, this);
        this.coldObservables.push(observable);
        return observable;
    };
    TestScheduler.prototype.createHotObservable = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var marbleValue = args[0], value = args[1], error = args[2];
        var messages = Array.isArray(marbleValue)
            ? marbleValue
            : parseObservableMarble_1.parseObservableMarble(marbleValue, value, error, false, this.frameTimeFactor);
        var subject = new HotObservable_1.HotObservable(messages, this);
        this.hotObservables.push(subject);
        return subject;
    };
    TestScheduler.prototype.materializeInnerObservable = function (observable, outerFrame) {
        var _this = this;
        var innerObservableMetadata = [];
        var pushMetaData = function (notification) {
            return innerObservableMetadata.push(new TestMessage_1.TestMessageValue(_this.frame - outerFrame, notification));
        };
        observable.subscribe(function (value) { return pushMetaData(Notification_1.Notification.createNext(value)); }, function (err) { return pushMetaData(Notification_1.Notification.createError(err)); }, function () { return pushMetaData(Notification_1.Notification.createComplete()); });
        return innerObservableMetadata;
    };
    return TestScheduler;
}(VirtualTimeScheduler_1.VirtualTimeScheduler));
exports.TestScheduler = TestScheduler;
//# sourceMappingURL=TestScheduler.js.map